﻿--[[	*** DataStore_Reputations ***
Written by : Thaoky, EU-Marécages de Zangar
June 22st, 2009
--]]
DataStore_Reputations = LibStub("AceAddon-3.0"):NewAddon("DataStore_Reputations", "AceConsole-3.0", "AceEvent-3.0")

local addon = DataStore_Reputations

local THIS_ACCOUNT = "Default"

-- Reputation data is not stored line by line as returned by the game's API, instead, it is stored in a common table (per realm) where data is grouped by reputation.
-- If client addons want a faction list as it appears in game, they only need to save the faction list, and not the data, since it's handled here. That's the whole point of this addon.
local AddonDB_Defaults = {
	global = {
		['*'] = {							-- Account: "Default" or user defined for other accounts
			['*'] = {						-- Realm
				Reputations = {
					['*'] = {				-- "Ironforge"
						['*'] = nil			-- Character name = value
					}
				},
			}
		}
	}
}

function addon:OnInitialize()
	addon.db = LibStub("AceDB-3.0"):New("DataStore_ReputationsDB", AddonDB_Defaults)
end

function addon:OnEnable()
	local realm = GetRealmName()
	local player = UnitName("player")
	
	self.ThisAccount = addon.db.global[THIS_ACCOUNT]
	self.ThisRealm = self.ThisAccount[realm]
	-- self.ThisCharacter = self.ThisRealm.Characters[player]

	self:RegisterEvent("PLAYER_ALIVE")
	self:RegisterEvent("CHAT_MSG_COMBAT_FACTION_CHANGE")
end

function addon:OnDisable()
	self:UnregisterEvent("PLAYER_ALIVE")
	self:UnregisterEvent("CHAT_MSG_COMBAT_FACTION_CHANGE")
end

-- *** Utility functions ***

-- *** Scanning functions ***
local function ScanReputations()
	local headersState = {} 	-- save the state of headers, restore it after scan
	
	for i = GetNumFactions(), 1, -1 do		-- 1st pass, expand all headers (from last to first), otherwise data can't be collected
		local name, _, _, _, _, _, _,	_, isHeader, isCollapsed = GetFactionInfo(i)
		if isHeader and isCollapsed then
			headersState[name] = isCollapsed
			ExpandFactionHeader(i)				-- triggers UPDATE_FACTION
		end
	end

	local r = addon.ThisRealm.Reputations
	local player = UnitName("player")
	
	for i = 1, GetNumFactions() do		-- 2nd pass, data collection
		local name, _, _, bottom, top, earned, _,	_, isHeader, _, hasRep = GetFactionInfo(i)
		if (not isHeader) or (isHeader and hasRep) then
			-- new in 3.0.2, headers may have rep, ex: alliance vanguard + horde expedition
			r[name][player] = bottom .. "|" .. top .. "|" .. earned
		end
	end
	
	-- restore headers
	for i = GetNumFactions(), 1, -1 do
		local name, _, _, _, _, _, _,	_, isHeader = GetFactionInfo(i)
		if isHeader and headersState[name] then
			CollapseFactionHeader(i)
		end
	end
end

-- *** DB functions ***
function addon:GetReputations(realm, account)
	-- assert(type(name) == "string")
	-- assert(type(realm) == "string")
	-- assert(type(account) == "string")
	
	-- Usage: 
	-- 	local c = addon:GetCharacter(char, realm, account)
	--	all 3 parameters default to current player, realm or account
	realm = realm or GetRealmName()
	account = account or THIS_ACCOUNT
	
	return addon.db.global[account][realm].Reputations
end

function addon:DeleteCharacter(name, realm, account)
	for _, repTable in pairs(addon:GetReputations(realm, account)) do
		for characterName, _ in pairs(repTable) do
			if characterName == name then
				repTable[characterName] = nil
			end
		end
	end
end

local BottomLevels = {
	[-42000] = FACTION_STANDING_LABEL1,	 -- "Hated"
	[-6000] = FACTION_STANDING_LABEL2,	 -- "Hostile"
	[-3000] = FACTION_STANDING_LABEL3,	 -- "Unfriendly"
	[0] = FACTION_STANDING_LABEL4,		 -- "Neutral"
	[3000] = FACTION_STANDING_LABEL5,	 -- "Friendly"
	[9000] = FACTION_STANDING_LABEL6,	 -- "Honored"
	[21000] = FACTION_STANDING_LABEL7,	 -- "Revered"
	[42000] = FACTION_STANDING_LABEL8,	 -- "Exalted"
}

function addon:GetReputationInfo(faction, name, realm, account)
	local r = addon:GetReputations(realm, account)
	
	local reputationData = r[faction][name]		-- Ex "3000|9000|7680"
	
	-- if any of the four parameters has no data in the db (bad account name, bad realm, bad alt, bad faction), just exit
	-- error checking is done automatically through ace db's magic wildcard
	if not reputationData then return end

	local bottom, top, earned = strsplit("|", reputationData)
	bottom = tonumber(bottom)
	top = tonumber(top)
	earned = tonumber(earned)
	local rate = (earned - bottom) / (top - bottom) * 100

	-- ex: "Revered", 15400, 21000, 73%
	return BottomLevels[bottom], (earned - bottom), (top - bottom), rate 
end

function addon:GetRawReputationInfo(faction, name, realm, account)
	-- same as addon:GetReputationInfo, but returns raw values
	local r = addon:GetReputations(realm, account)
	
	local reputationData = r[faction][name]		-- Ex "3000|9000|7680"
	
	-- if any of the four parameters has no data in the db (bad account name, bad realm, bad alt, bad faction), just exit
	-- error checking is done automatically through ace db's magic wildcard
	if not reputationData then return end

	local bottom, top, earned = strsplit("|", reputationData)
	return tonumber(bottom), tonumber(top), tonumber(earned)
end

local PT = LibStub("LibPeriodicTable-3.1")
local BF = LibStub("LibBabble-Faction-3.0"):GetUnstrictLookupTable()

function addon:GetSource(searchedID)
	-- returns the faction where a given item ID can be obtained, as well as the level
	local level, repData = PT:ItemInSet(searchedID, "Reputation.Reward")
	if level and repData then
		local _, _, faction = strsplit(".", repData)		-- ex: "Reputation.Reward.Sporeggar"
		faction = BF[faction] or faction		-- localize faction name if possible
	
		-- level = 7,  29150:7 where 7 means revered
		return faction, _G["FACTION_STANDING_LABEL"..level]
	end
end

-- *** EVENT HANDLERS ***
function addon:PLAYER_ALIVE()
	ScanReputations()
end

-- this turns
--	"Reputation with %s increased by %d."
-- into
--	"Reputation with (%a+) increased by (%d+)."
local repUpMsg = gsub(FACTION_STANDING_INCREASED, "%%s", "(%%a+)")
repUpMsg = gsub(repUpMsg, "%%d", "(%%d+)")

function addon:CHAT_MSG_COMBAT_FACTION_CHANGE(event, text)
	-- This code is a bit more complex than calling ScanReputations again.
	-- The purpose is to avoid triggering UPDATE_FACTION, as this may result in heavy processing in other addons
	if not text then return end
	
	local faction, value = text:match(repUpMsg)
	if faction and value then
		local player = UnitName("player")
		local bottom, top, earned = addon:GetRawReputationInfo(faction, player)
		
		if earned then
			local newValue = earned + tonumber(value)
			if newValue >= top then	-- rep status increases (to revered, etc..)
				ScanReputations()					-- so scan all
			else
				addon.ThisRealm.Reputations[faction][player] = bottom .. "|" .. top .. "|" .. newValue
			end
		end
	end
end
